package com.tourism.webadmin.app.website.controller;

import cn.dev33.satoken.annotation.SaIgnore;
import com.tourism.common.additional.utils.UrlConvertUtils;
import com.tourism.common.core.object.MyOrderParam;
import com.tourism.common.core.object.MyPageData;
import com.tourism.common.core.object.ResponseResult;
import com.tourism.common.core.util.MyModelUtil;
import com.tourism.common.log.annotation.OperationLog;
import com.tourism.common.log.model.constant.SysOperationLogType;
import com.tourism.webadmin.app.website.dto.TourismProjectTravelNotesToWebDto;
import com.tourism.webadmin.app.website.service.BasicToWebService;
import com.tourism.webadmin.app.website.vo.TourTravelNotesDirectoryCountryVo;
import com.tourism.webadmin.app.website.vo.TourTravelNotesDirectoryVo;
import com.tourism.webadmin.back.model.*;
import com.tourism.webadmin.back.service.*;
import com.tourism.webadmin.back.vo.TourTourismProjectTravelNotesVo;
import com.tourism.webadmin.back.vo.TourismProjectVo;
import com.tourism.common.additional.config.ApplicationConfig;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import java.util.ArrayList;
import java.util.HashSet;
import java.util.List;
import java.util.stream.Collectors;

@Tag(name = "门户网站旅游游记接口")
@Slf4j
@RestController
@SaIgnore
@Validated
@RequestMapping("/website/tourism/projectTravelNotes")
public class WebsiteTourismProjectTravelNotesController {

    @Autowired
    private BasicToWebService basicToWebServicel;
    @Autowired
    private DirectoryInfoService directoryInfoService;
    @Autowired
    private TourTravelNotesProjectRelationService tourTravelNotesProjectRelationService;
    @Autowired
    private TourismProjectService tourismProjectService;
    @Autowired
    private TourTourismProjectTravelNotesService tourTourismProjectTravelNotesService;
    @Autowired
    private ApplicationConfig applicationConfig;


    /**
     * 列出旅游游记列表。(首页)
     * @return 应答结果对象，包含查询结果集。
     */
    @OperationLog(type = SysOperationLogType.LIST)
    @GetMapping("/homeList")
    public ResponseResult<MyPageData<TourTourismProjectTravelNotesVo>> tourTourismTravelNotesList(TourismProjectTravelNotesToWebDto tourismProjectTravelNotesToWebDto) {
        MyPageData<TourTourismProjectTravelNotesVo> tourTourismProjectTravelNotesList = basicToWebServicel.getTourTourismProjectTravelNotesList(tourismProjectTravelNotesToWebDto);

        return ResponseResult.success(tourTourismProjectTravelNotesList);
    }

    /**
     * 列出符合过滤条件的旅游游记列表。(游记页)
     * @return 应答结果对象，包含查询结果集。
     */
    @OperationLog(type = SysOperationLogType.LIST)
    @GetMapping("/travelNotesPageList")
    public ResponseResult<MyPageData<TourTourismProjectTravelNotesVo>> travelNotesPageList(TourismProjectTravelNotesToWebDto tourismProjectTravelNotesToWebDto) {
        MyPageData<TourTourismProjectTravelNotesVo> tourTourismProjectTravelNotesList = basicToWebServicel.getTravelNotesPageList(tourismProjectTravelNotesToWebDto);

        return ResponseResult.success(tourTourismProjectTravelNotesList);
    }

    /**
     * 通过旅游游记的id查询游记的详情。
     * @return 应答结果对象，包含查询结果集。
     */
    @OperationLog(type = SysOperationLogType.LIST)
    @GetMapping("/travelNotesDetail")
    public ResponseResult<TourTourismProjectTravelNotesVo> travelNotesDetail(@RequestParam Long id) {

        return ResponseResult.success(basicToWebServicel.travelNotesDetail(id));
    }

    /**
     * 用户游记点赞
     * param travelNotesId 游记id
     * param type 0取消点赞 1点赞
     * @return 应答结果对象，包含查询结果集。
     */
    @OperationLog(type = SysOperationLogType.UPDATE)
    @GetMapping("/userLikeTravelNotesUpdate")
    public ResponseResult<Void> userLikeTravelNotesUpdate(@RequestParam Long travelNotesId,Integer type) {

        basicToWebServicel.userLikeTravelNotesUpdate(travelNotesId,type);
        return ResponseResult.success();
    }

    /**
     * 用户是否对该游记进行点赞
     * param travelNotesId 游记id
     * @return 应答结果对象，包含查询结果集。
     */
    @OperationLog(type = SysOperationLogType.LIST)
    @GetMapping("/isLikeTravelNotes")
    public ResponseResult<Boolean> isLikeTravelNotes(@RequestParam Long travelNotesId) {
        return ResponseResult.success(basicToWebServicel.isLikeTravelNotes(travelNotesId));
    }


    /**
     * 用户游记增加浏览量
     * param travelNotesId 游记id
     * @return 应答结果对象，包含查询结果集。
     */
    @OperationLog(type = SysOperationLogType.ADD)
    @GetMapping("/travelNotesViewCountAdd")
    public ResponseResult<Void> travelNotesViewCountAdd(@RequestParam Long travelNotesId) {
        basicToWebServicel.travelNotesViewCountAdd(travelNotesId);
        return ResponseResult.success();
    }


    /**
     * 游记菜单列表(二维数组)
     * param
     * @return 应答结果对象，包含查询结果集。
     */
    @OperationLog(type = SysOperationLogType.LIST)
    @GetMapping("/travelNotesDirectoryList")
    public ResponseResult<List<TourTravelNotesDirectoryVo>> travelNotesDirectoryList() {

        //设置排序
        MyOrderParam myOrderParam = new MyOrderParam();
        MyOrderParam.OrderInfo orderInfo = new MyOrderParam.OrderInfo();
        orderInfo.setAsc(true);
        orderInfo.setFieldName("showOrder");
        myOrderParam.add(orderInfo);
        String orderBy = MyOrderParam.buildOrderBy(myOrderParam, DirectoryInfo.class);

        //查询一级菜单
        DirectoryInfo directoryInfo = new DirectoryInfo();
        directoryInfo.setParentId(0L);
        directoryInfo.setEnable(1);
        directoryInfo.setDirectoryType(1);
        List<DirectoryInfo> directoryInfoList = directoryInfoService.getDirectoryInfoList(directoryInfo, orderBy);
        List<TourTravelNotesDirectoryVo> tourTravelNotesDirectoryVoList= new ArrayList<>();
        directoryInfoList.stream().forEach(item->{
            TourTravelNotesDirectoryVo tourTravelNotesDirectoryVo = new TourTravelNotesDirectoryVo();
            tourTravelNotesDirectoryVo.setAreaId(item.getId().toString());
            tourTravelNotesDirectoryVo.setAreaName(item.getMenuName());

            DirectoryInfo directoryInfo1 = new DirectoryInfo();
            directoryInfo1.setEnable(1);
            directoryInfo1.setParentId(item.getId());
            directoryInfo1.setDirectoryType(1);
            List<DirectoryInfo> countryList = directoryInfoService.getDirectoryInfoList(directoryInfo1,orderBy);
            List<TourTravelNotesDirectoryCountryVo> tourTravelNotesDirectoryCountryVoList = new ArrayList<>();
            for(DirectoryInfo country:countryList){
                TourTravelNotesDirectoryCountryVo tourTravelNotesDirectoryCountryVo = new TourTravelNotesDirectoryCountryVo();
                tourTravelNotesDirectoryCountryVo.setCountryId(country.getId().toString());
                tourTravelNotesDirectoryCountryVo.setCountryName(country.getMenuName().toString());
                tourTravelNotesDirectoryCountryVoList.add(tourTravelNotesDirectoryCountryVo);
            }
            tourTravelNotesDirectoryVo.setChildren(tourTravelNotesDirectoryCountryVoList);
            tourTravelNotesDirectoryVoList.add(tourTravelNotesDirectoryVo);
        });
        return ResponseResult.success(tourTravelNotesDirectoryVoList);
    }

    /**
     * 相关项目
     * param travelNotesId 游记id
     * @return 应答结果对象，包含查询结果集。
     */
    @OperationLog(type = SysOperationLogType.LIST)
    @GetMapping("/relateProjectList")
    public ResponseResult<List<TourismProjectVo>> relateProjectList(@RequestParam(required = true) Long travelNotesId) {

        TourTravelNotesProjectRelation tourTravelNotesProjectRelation = new TourTravelNotesProjectRelation();
        tourTravelNotesProjectRelation.setTravelNotesId(travelNotesId);
        List<TourTravelNotesProjectRelation> tourTravelNotesProjectRelationList =
                tourTravelNotesProjectRelationService.getTourTravelNotesProjectRelationList(tourTravelNotesProjectRelation, "");
        ArrayList<Long> relateProjectList =
                tourTravelNotesProjectRelationList.stream().map(TourTravelNotesProjectRelation::getProjectId).collect(Collectors.toCollection(ArrayList::new));

        if(relateProjectList.size() == 0) {
            return ResponseResult.success(new ArrayList<>());
        }else {
            List<TourismProject> tourismProjectList = tourismProjectService.getInList(new HashSet<>(relateProjectList));
            List<TourismProjectVo> tourismProjectVoList = MyModelUtil.copyCollectionTo(tourismProjectList, TourismProjectVo.class);
            //先把imgUrl由jaon转换为List<FileUrlObject>
            tourismProjectVoList.stream().forEach(item ->
            {
                List<String> urlList = UrlConvertUtils.urlConvert(applicationConfig.getHostIpPort(), item.getHomeHotPicture());
                item.setHomeHotPicturesAfterConvert(urlList);
                List<String> arrayList = UrlConvertUtils.urlConvert(applicationConfig.getHostIpPort(), item.getTourismUrl());
                item.setTourismUrlsAfterConvert(arrayList);
                List<String> arrayList1 = UrlConvertUtils.urlConvert(applicationConfig.getHostIpPort(), item.getTravelNotesBanner());
                item.setTravelNotesBannerAfterConvert(arrayList1);
            });
            return ResponseResult.success(tourismProjectVoList);
        }
    }



    /**
     * 获取点赞数量
     * param travelNotesId 游记id
     * @return 应答结果对象，包含查询结果集。
     */
    @OperationLog(type = SysOperationLogType.LIST)
    @GetMapping("/getLikeCount")
    public ResponseResult<Integer> getLikeCount(@RequestParam(required = true) Long travelNotesId) {

        TourTourismProjectTravelNotes tourTourismProjectTravelNotes = tourTourismProjectTravelNotesService.getById(travelNotesId);
        Integer likeCount = tourTourismProjectTravelNotes.getLikeCount();
        return ResponseResult.success(likeCount);
    }
}
