package com.tourism.webadmin.app.website.service.impl;

import com.github.pagehelper.page.PageMethod;
import com.tourism.common.additional.utils.StringUtils;
import com.tourism.common.additional.utils.UrlConvertUtils;
import com.tourism.common.core.object.MyOrderParam;
import com.tourism.common.core.object.MyPageData;
import com.tourism.common.core.object.MyRelationParam;
import com.tourism.common.core.object.TokenData;
import com.tourism.common.core.util.MyModelUtil;
import com.tourism.common.core.util.MyPageUtil;
import com.tourism.webadmin.app.website.dto.TourismProjectTravelNotesToWebDto;
import com.tourism.webadmin.app.website.service.BasicToWebService;
import com.tourism.webadmin.back.model.BannerInfo;
import com.tourism.webadmin.back.model.DirectoryInfo;
import com.tourism.webadmin.back.model.TourTourismProjectTravelNotes;
import com.tourism.webadmin.back.model.TourUserLikeTravelNotes;
import com.tourism.webadmin.back.service.BannerInfoService;
import com.tourism.webadmin.back.service.DirectoryInfoService;
import com.tourism.webadmin.back.service.TourTourismProjectTravelNotesService;
import com.tourism.webadmin.back.service.TourUserLikeTravelNotesService;
import com.tourism.webadmin.back.vo.BannerInfoVo;
import com.tourism.webadmin.back.vo.DirectoryInfoVo;
import com.tourism.webadmin.back.vo.TourTourismProjectTravelNotesVo;
import com.tourism.common.additional.config.ApplicationConfig;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.List;
import java.util.Random;
import java.util.stream.Collectors;

/**
 * 门户网站首页Service
 * 
 * @author tourism
 */
@Slf4j
@Service("BasicToWebService")
public class BasicToWebServiceImpl implements BasicToWebService
{

    @Autowired
    private BannerInfoService bannerInfoService;
    @Autowired
    private DirectoryInfoService directoryInfoService;
    @Autowired
    private ApplicationConfig applicationConfig;
    @Autowired
    private TourTourismProjectTravelNotesService tourTourismProjectTravelNotesService;
    @Autowired
    private TourUserLikeTravelNotesService tourUserLikeTravelNotesService;

    @Override
    public MyPageData<BannerInfoVo> getBannerInfoList(Long belongTab) {
        BannerInfo bannerInfoFilter = new BannerInfo();
        bannerInfoFilter.setBelongTab(belongTab);
        //首页展示的话，只能展示启用的banner图
        bannerInfoFilter.setEnable(1);
        MyOrderParam.OrderInfo orderInfo = new MyOrderParam.OrderInfo();
        orderInfo.setFieldName("mainBanner");
        orderInfo.setAsc(false);
        MyOrderParam myOrderParam  = new MyOrderParam();
        myOrderParam.add(orderInfo);
        String orderBy = MyOrderParam.buildOrderBy(myOrderParam, BannerInfo.class);
        List<BannerInfo> bannerInfoList = bannerInfoService.getBannerInfoListWithRelation(bannerInfoFilter, orderBy);
        MyPageData<BannerInfoVo> bannerInfoVoMyPageData = MyPageUtil.makeResponseData(bannerInfoList, BannerInfoVo.class);

        List<BannerInfoVo> dataList = bannerInfoVoMyPageData.getDataList();
            //先把imgUrl由jaon转换为List<FileUrlObject>
        dataList.stream().forEach(item ->{
            if (StringUtils.isNotEmpty(item.getImgUrl())) {
                item.setImgUrlsAfterConvert(UrlConvertUtils.urlConvert(applicationConfig.getHostIpPort(), item.getImgUrl()));
            }
        });

        return bannerInfoVoMyPageData;
    }

    @Override
    public MyPageData<DirectoryInfoVo> getDirectoryInfoList(Long parentId, Integer isHotspot, Integer isAll) {

        DirectoryInfo directoryInfoFilter = new DirectoryInfo();
        //当directoryId为1000时,则表明查询三级菜单
        if(parentId != null){
            directoryInfoFilter.setParentId(parentId);
        }
        if(isAll!= null && isAll == 1){
            directoryInfoFilter.setDirectoryId(1000L);
            //菜单类型：1：旅游，2：劳务
            directoryInfoFilter.setDirectoryType(1);
        }
        if(isHotspot != null){
            directoryInfoFilter.setIsHotspot(isHotspot);
        }
        MyOrderParam myOrderParam = new MyOrderParam();
        MyOrderParam.OrderInfo orderInfo = new MyOrderParam.OrderInfo();
        //先根据是否热点排序，再根据展示顺序排序
        orderInfo.setFieldName("isHotspot");
        orderInfo.setAsc(false);
        myOrderParam.add(orderInfo);
        orderInfo.setFieldName("showOrder");
        orderInfo.setAsc(true);
        myOrderParam.add(orderInfo);
        String orderBy = MyOrderParam.buildOrderBy(myOrderParam, DirectoryInfo.class);
        //看到的项目为启用的项目
        directoryInfoFilter.setEnable(1);
        List<DirectoryInfo> directoryInfoList =
                directoryInfoService.getDirectoryInfoListWithRelation(directoryInfoFilter, orderBy);
//        List<DirectoryInfo> directoryInfoList =
//                directoryInfoService.getDirectoryInfoListWithRelation(directoryInfoFilter, orderBy);

        MyPageData<DirectoryInfoVo> directoryInfoVoMyPageData = MyPageUtil.makeResponseData(directoryInfoList, DirectoryInfoVo.class);

        List<DirectoryInfoVo> dataList = directoryInfoVoMyPageData.getDataList();
        //先把imgUrl由jaon转换为List<FileUrlObject>
        dataList.stream().forEach(item ->{
            if (StringUtils.isNotEmpty(item.getHotPictureUrl())) {
                item.setHotPictureUrlsAfterConvert(UrlConvertUtils.urlConvert(applicationConfig.getHostIpPort(), item.getHotPictureUrl()));
            }
        });
        return directoryInfoVoMyPageData;
    }

    @Override
    public MyPageData<TourTourismProjectTravelNotesVo> getTourTourismProjectTravelNotesList(TourismProjectTravelNotesToWebDto tourismProjectTravelNotesToWebDto){

        TourTourismProjectTravelNotes tourTourismProjectTravelNotesFilter = new TourTourismProjectTravelNotes();
        tourTourismProjectTravelNotesFilter.setEnable(1);
        tourTourismProjectTravelNotesFilter.setIsHotspot(1);
        if(tourismProjectTravelNotesToWebDto.getPageNum() != null && tourismProjectTravelNotesToWebDto.getPageSize() != null) {
            PageMethod.startPage(tourismProjectTravelNotesToWebDto.getPageNum(), tourismProjectTravelNotesToWebDto.getPageSize(), true);
        }
        MyOrderParam myOrderParam = new MyOrderParam();
        myOrderParam.add(new MyOrderParam.OrderInfo("showOrder",true,null));
        String orderBy = MyOrderParam.buildOrderBy(myOrderParam, TourTourismProjectTravelNotes.class);
        List<TourTourismProjectTravelNotes> tourTourismProjectTravelNotesList = tourTourismProjectTravelNotesService.getTourTourismProjectTravelNotesList(tourTourismProjectTravelNotesFilter, orderBy);
        MyPageData<TourTourismProjectTravelNotesVo> tourTourismProjectTravelNotesVoMyPageData = MyPageUtil.makeResponseData(tourTourismProjectTravelNotesList, TourTourismProjectTravelNotesVo.class);
        List<TourTourismProjectTravelNotesVo> dataList = tourTourismProjectTravelNotesVoMyPageData.getDataList();
        dataList.stream().forEach(item->{
            if (StringUtils.isNotEmpty(item.getHomeHotPicture())) {
                item.setHomeHotPicturesAfterConvert(UrlConvertUtils.urlConvert(applicationConfig.getHostIpPort(), item.getHomeHotPicture()));
            }
        });

        return tourTourismProjectTravelNotesVoMyPageData;
    }

    @Override
    public MyPageData<TourTourismProjectTravelNotesVo> getTravelNotesPageList(TourismProjectTravelNotesToWebDto tourismProjectTravelNotesToWebDto){
        //当belongTab为1的时候，则表明查询全部的游记
        MyOrderParam myOrderParam = new MyOrderParam();
        myOrderParam.add(new MyOrderParam.OrderInfo("isHotspot",false,null));
        myOrderParam.add(new MyOrderParam.OrderInfo("showOrder",true,null));
        String orderBy = MyOrderParam.buildOrderBy(myOrderParam, TourTourismProjectTravelNotes.class);

        TourTourismProjectTravelNotes tourTourismProjectTravelNotes = new TourTourismProjectTravelNotes();

        if(StringUtils.isEmpty(tourismProjectTravelNotesToWebDto.getAreaId()) && StringUtils.isEmpty(tourismProjectTravelNotesToWebDto.getCountryId())){
            //判断是否为全部的地区
            tourTourismProjectTravelNotes.setEnable(1);
            tourTourismProjectTravelNotes.setBelongTab(null);
        }else if(StringUtils.isNotEmpty(tourismProjectTravelNotesToWebDto.getAreaId()) && StringUtils.isEmpty(tourismProjectTravelNotesToWebDto.getCountryId())){
            //判断是否为该地区的所有国家
            DirectoryInfo directoryInfo = new DirectoryInfo();
            directoryInfo.setParentId(Long.parseLong(tourismProjectTravelNotesToWebDto.getAreaId()));
            directoryInfo.setEnable(1);
            List<DirectoryInfo> directoryInfoList =
                    directoryInfoService.getDirectoryInfoList(directoryInfo, "");
            List<Long> directoryIdList =
                    directoryInfoList.stream().map(DirectoryInfo::getId).collect(Collectors.toList());
            tourTourismProjectTravelNotes.setBelongTab(null);
            tourTourismProjectTravelNotes.setDirectoryInfoIds(directoryIdList);
        }else {
            //如果查询的是某个国家的游记的话
            if(StringUtils.isNotEmpty(tourismProjectTravelNotesToWebDto.getCountryId())) {
                tourTourismProjectTravelNotes.setBelongTab(Long.parseLong(tourismProjectTravelNotesToWebDto.getCountryId()));
            }
        }

        if(tourismProjectTravelNotesToWebDto.getPageNum() != null && tourismProjectTravelNotesToWebDto.getPageSize() != null) {
            PageMethod.startPage(tourismProjectTravelNotesToWebDto.getPageNum(), tourismProjectTravelNotesToWebDto.getPageSize(), true);
        }

        List<TourTourismProjectTravelNotes> tourTourismProjectTravelNotesList =
                tourTourismProjectTravelNotesService.getTourTourismProjectTravelNotesList(tourTourismProjectTravelNotes, orderBy);

        MyPageData<TourTourismProjectTravelNotesVo> tourTourismProjectTravelNotesVoMyPageData =
                MyPageUtil.makeResponseData(tourTourismProjectTravelNotesList, TourTourismProjectTravelNotesVo.class);

        List<TourTourismProjectTravelNotesVo> dataList = tourTourismProjectTravelNotesVoMyPageData.getDataList();

        dataList.stream().forEach(item->{
            if(StringUtils.isNotEmpty(item.getTourismUrl())){
                item.setTourismUrlsAfterConvert(UrlConvertUtils.urlConvert(applicationConfig.getHostIpPort(), item.getTourismUrl()));
            }
        });

        return tourTourismProjectTravelNotesVoMyPageData;
    }

    @Override
    public TourTourismProjectTravelNotesVo travelNotesDetail(Long id){
        TourTourismProjectTravelNotes tourTourismProjectTravelNotes =
                tourTourismProjectTravelNotesService.getByIdWithRelation(id, MyRelationParam.full());

        TourTourismProjectTravelNotesVo tourTourismProjectTravelNotesVo =
                MyModelUtil.copyTo(tourTourismProjectTravelNotes, TourTourismProjectTravelNotesVo.class);

        if(tourTourismProjectTravelNotesVo != null) {
            if (StringUtils.isNotEmpty(tourTourismProjectTravelNotesVo.getTourismUrl())) {
                tourTourismProjectTravelNotesVo.setTourismUrlsAfterConvert(UrlConvertUtils.urlConvert(applicationConfig.getHostIpPort(), tourTourismProjectTravelNotesVo.getTourismUrl()));
            }

            if (StringUtils.isNotEmpty(tourTourismProjectTravelNotesVo.getHomeHotPicture())) {
                tourTourismProjectTravelNotesVo.setHomeHotPicturesAfterConvert(UrlConvertUtils.urlConvert(applicationConfig.getHostIpPort(), tourTourismProjectTravelNotesVo.getHomeHotPicture()));
            }

            if (StringUtils.isNotEmpty(tourTourismProjectTravelNotesVo.getTravelNotesBanner())) {
                tourTourismProjectTravelNotesVo.setTravelNotesBannerAfterConvert(UrlConvertUtils.urlConvert(applicationConfig.getHostIpPort(), tourTourismProjectTravelNotesVo.getTravelNotesBanner()));
            }
        }
        return tourTourismProjectTravelNotesVo;
    }

    @Override
    @Transactional
    public void userLikeTravelNotesUpdate(Long travelNotesId,Integer type){

        TourUserLikeTravelNotes tourUserLikeTravelNotes =
                new TourUserLikeTravelNotes();
        tourUserLikeTravelNotes.setTravelNotesId(travelNotesId);
        tourUserLikeTravelNotes.setUserId(TokenData.takeFromRequest().getUserId());
        TourTourismProjectTravelNotes tourTourismProjectTravelNotes = tourTourismProjectTravelNotesService.getById(travelNotesId);
        if(tourTourismProjectTravelNotes.getLikeCount() != null) {
            if(type == 1) {
                tourTourismProjectTravelNotes.setLikeCount(tourTourismProjectTravelNotes.getLikeCount() + 1);
                if(tourTourismProjectTravelNotes.getHotValue() != null) {
                    tourTourismProjectTravelNotes.setHotValue(tourTourismProjectTravelNotes.getHotValue() + 100);
                }else {
                    tourTourismProjectTravelNotes.setHotValue(100);
                }
            }else {
                tourTourismProjectTravelNotes.setLikeCount(tourTourismProjectTravelNotes.getLikeCount() - 1);
            }
        }else {
            tourTourismProjectTravelNotes.setLikeCount(1);
        }
        if(type == 0){
            tourUserLikeTravelNotesService.removeBy(tourUserLikeTravelNotes);
        }else {
            tourUserLikeTravelNotesService.saveNew(tourUserLikeTravelNotes);
        }
        tourTourismProjectTravelNotesService.updateById(tourTourismProjectTravelNotes);
    }

    @Override
    public Boolean isLikeTravelNotes(Long travelNotesId){
        TourUserLikeTravelNotes tourUserLikeTravelNotes = new TourUserLikeTravelNotes();
        tourUserLikeTravelNotes.setTravelNotesId(travelNotesId);
        tourUserLikeTravelNotes.setUserId(TokenData.takeFromRequest().getUserId());
        return tourUserLikeTravelNotesService.existByFilter(tourUserLikeTravelNotes);

    }

    @Override
    public void travelNotesViewCountAdd(Long travelNotesId){

        TourTourismProjectTravelNotes tourTourismProjectTravelNotes = tourTourismProjectTravelNotesService.getById(travelNotesId);
        if(tourTourismProjectTravelNotes.getPageViewCount() != null) {
            tourTourismProjectTravelNotes.setPageViewCount(tourTourismProjectTravelNotes.getPageViewCount() + 1);
        }else {
            tourTourismProjectTravelNotes.setPageViewCount(1);
        }
        //每有一人浏览，则增加50热度值
        if(tourTourismProjectTravelNotes.getHotValue() != null) {
            tourTourismProjectTravelNotes.setHotValue(tourTourismProjectTravelNotes.getHotValue() + 50);
        }else {
            tourTourismProjectTravelNotes.setHotValue(50);
        }
        tourTourismProjectTravelNotesService.updateById(tourTourismProjectTravelNotes);
    }

    @Override
    public void generate(){
        //随机生成1k-2w的浏览量,浏览量除以2为（点赞数的上限)浏览量除以3为(点赞数的下限),热度值为浏览量的二倍以上-三倍以下随机数,这三个数值都为整数
        List<TourTourismProjectTravelNotes> tourTourismProjectTravelNotesList =
                tourTourismProjectTravelNotesService.getTourTourismProjectTravelNotesList(new TourTourismProjectTravelNotes(), "");

        tourTourismProjectTravelNotesList.stream().forEach(item->{

            Random random = new Random();
            // 生成1k-2w的浏览量
            int minViews = 1000;
            int maxViews = 20000;
            int views = random.nextInt(maxViews - minViews + 1) + minViews;

            // 计算点赞数的上限和下限
            int maxLikes = views / 3;
            int minLikes = views / 4;
            int likes = random.nextInt(maxLikes - minLikes + 1) + minLikes;

            // 生成热度值（）
            int heat = (views*50)+(likes*100);

            item.setHotValue(heat);
            item.setLikeCount(likes);
            item.setPageViewCount(views);

            tourTourismProjectTravelNotesService.updateById(item);
        });
    }

}

